@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- * libmm -- Access modem status & information from glib applications
+ * libmm-glib -- Access modem status & information from glib applications
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -62,10 +62,7 @@ struct _MMSmsClass {
 };
 
 GType mm_sms_get_type (void);
-
-#if GLIB_CHECK_VERSION(2, 44, 0)
 G_DEFINE_AUTOPTR_CLEANUP_FUNC (MMSms, g_object_unref)
-#endif
 
 const gchar *mm_sms_get_path                     (MMSms *self);
 gchar       *mm_sms_dup_path                     (MMSms *self);
